@@ -11,7 +11,12 @@ common to applications.
 :license: MIT (See: LICENSE)
 """
 
-from circuits.app.log import Logger
+from circuits.app.log import Log, Logger
+
 from circuits.app.daemon import Daemon
-from circuits.app.config import Config
-from circuits.app.env import Environment
+
+from circuits.app.config import Config, LoadConfig, SaveConfig
+
+from circuits.app.env import BaseEnvironment
+from circuits.app.env import UpgradeEnvironment
+from circuits.app.env import CreateEnvironment, LoadEnvironment
