@@ -12,10 +12,10 @@ import sys
 from cStringIO import StringIO
 
 from handlers import handler
-from components import Component
+from components import BaseComponent
 from circuits.tools import reprhandler
 
-class Debugger(Component):
+class Debugger(BaseComponent):
     """Create a new Debugger Component
 
     Creates a new Debugger Component that filters all events in teh system
@@ -53,8 +53,8 @@ class Debugger(Component):
         self.IgnoreEvents.extend(kwargs.get("IgnoreEvents", []))
         self.IgnoreChannels.extend(kwargs.get("IgnoreChannels", []))
 
-    @handler("exception", filter=True)
-    def exception(self, type, value, traceback, handler=None):
+    @handler("exception", priority=100.0)
+    def _on_exception(self, type, value, traceback, handler=None):
         if not self.errors:
             return
 
@@ -78,8 +78,8 @@ class Debugger(Component):
 
         s.close()
 
-    @handler(filter=True)
-    def event(self, event, *args, **kwargs):
+    @handler(priority=100.0)
+    def _on_event(self, event, *args, **kwargs):
         """Global Event Handler
 
         Event handler to listen and filter all events printing each event
@@ -96,11 +96,11 @@ class Debugger(Component):
         elif channel in self.IgnoreChannels:
             return
         else:
+            s = repr(event)
+            if self.chop:
+                s = "%s ...>" % s[:75]
             if self.logger is not None:
-                self.logger.debug(repr(event))
+                self.logger.debug(s)
             else:
-                s = repr(event)
-                if self.file is sys.stderr and len(s) > 80 and self.chop:
-                    s = "%s ...>" % s[:75]
                 self.file.write("%s\n" % s)
                 self.file.flush()
