@@ -12,13 +12,8 @@ is used independently it should not be registered as it causes it's
 main event handler ``_on_task`` to execute in the other thread blocking it.
 """
 
-from time import time
-from uuid import uuid4 as uuid
-from random import seed, choice
-
 from circuits.core import handler, BaseComponent, Event
 
-seed(time())
 
 class Task(Event):
     """Task Event
@@ -41,6 +36,7 @@ class Task(Event):
 
         super(Task, self).__init__(f, *args, **kwargs)
 
+
 class Worker(BaseComponent):
 
     channel = "worker"
