@@ -4,8 +4,8 @@ import os
 from signal import SIGTERM
 from subprocess import Popen
 
-import py
-py.test.skip("XXX: Not passing...")
+import pytest
+pytest.importorskip("pytest_cov")
 
 from tests.app import app
 
