@@ -38,13 +38,13 @@ def test():
     assert e.future == False
     x = app.push(e)
 
-    pytest.wait_for(e, "future", True)
+    assert pytest.wait_for(e, "future", True)
     assert e.future == True
 
     def test(obj, attr):
         return isinstance(obj._value, list)
 
-    pytest.wait_for(x, None, test)
+    assert pytest.wait_for(x, None, test)
 
     assert x[0] == "Hello from %s" % pid
     assert x[1].startswith("Hello from")
