@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 
 import pytest
+pytest.skip()
 
 from circuits.core.manager import HAS_MULTIPROCESSING
 if not HAS_MULTIPROCESSING:
@@ -23,6 +24,8 @@ def test():
     app = App()
     app.start(link=m, process=True)
 
+    assert pytest.wait_for(app._bridge, "ready")
+
     x = m.push(Hello())
 
     assert pytest.wait_for(x, "result")
