@@ -4,11 +4,11 @@ class Client(Component):
 
     channel = "client"
 
-    def __init__(self):
-        super(Client, self).__init__()
+    def __init__(self, channel=channel):
+        super(Client, self).__init__(channel=channel)
 
         self.data = ""
-        self.ready = True
+        self.ready = False
         self.connected = False
         self.disconnected = False
 
